@@ -1,5 +1,5 @@
 # This file is part of tdmclient.
-# Copyright 2021 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
+# Copyright 2021-2022 ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE,
 # Miniature Mobile Robots group, Switzerland
 # Author: Yves Piguet
 #
@@ -9,7 +9,6 @@
 Python-to-Aseba transpiler (actually a tiny, yet useful subset of Python)
 """
 
-import sys
 import ast
 
 
@@ -60,6 +59,9 @@ class Context:
         # set of variables declared as global
         self.global_var = set()
 
+        # whether compiling to collect local variables, or final phase where all variables are known
+        self.collect_local_variables = False
+
         # function definitions indexed by function name (nodes ast.FunctionDef)
         self.functions = {}
 
@@ -88,12 +90,26 @@ class Context:
         """
         return name not in self.var
 
-    def var_str(self, name, is_target=False):
+    def var_str(self, name, is_target=False, ast_node=None):
         """Convert a variable name to its string representation in output
         source code.
         """
         is_global = self.is_global(name)
         context = self.parent_context if is_global and self.parent_context is not None else self
+
+        # check that constant is not assigned to
+        if not self.collect_local_variables and is_target:
+            if "." in name:
+                module_name, symbol = name.split(".", 1)
+                module_and_name = context.get_module(module_name), symbol
+            else:
+                module_and_name = context.get_module_for_symbol(name)
+            if module_and_name is not None:
+                module, local_name = module_and_name
+                if local_name in module.constants:
+                    raise TranspilerError(f"assignment to constant {name}",
+                                          ast_node=ast_node)
+
         module_value = context.get_module_value(name, not is_target)
         if module_value:
             return module_value
@@ -196,8 +212,8 @@ class Context:
             if module_and_name is not None:
                 module, name = module_and_name
                 return (module.constants[name][1] if name in module.constants
-                    else module.variables[name][1] if name in module.variables
-                    else False)
+                        else module.variables[name][1] if name in module.variables
+                        else False)
             var = (ATranspiler.PREDEFINED_VARIABLES if name in ATranspiler.PREDEFINED_VARIABLES
                    else self.parent_context.var if (name in self.global_var or not is_target and name not in self.var) and self.parent_context is not None
                    else self.var)
@@ -564,18 +580,32 @@ class ATranspiler:
         elif isinstance(node, ast.Num):
             code = f"{node.n:d}"
         elif isinstance(node, ast.BinOp):
-            op_str, priority = {
-                ast.Add: ("+", self.PRI_ADD),
-                ast.BitAnd: ("&", self.PRI_BINARY_AND),
-                ast.BitOr: ("|", self.PRI_BINARY_OR),
-                ast.BitXor: ("^", self.PRI_BINARY_XOR),
-                ast.FloorDiv: ("/", self.PRI_MULT),
-                ast.LShift: ("<<", self.PRI_SHIFT),
-                ast.Mod: ("%", self.PRI_MOD),
-                ast.Mult: ("*", self.PRI_MULT),
-                ast.RShift: (">>", self.PRI_SHIFT),
-                ast.Sub: ("-", self.PRI_ADD),
-            }[type(node.op)]
+            try:
+                op_str, priority = {
+                    ast.Add: ("+", self.PRI_ADD),
+                    ast.BitAnd: ("&", self.PRI_BINARY_AND),
+                    ast.BitOr: ("|", self.PRI_BINARY_OR),
+                    ast.BitXor: ("^", self.PRI_BINARY_XOR),
+                    ast.FloorDiv: ("/", self.PRI_MULT),
+                    ast.LShift: ("<<", self.PRI_SHIFT),
+                    ast.Mod: ("%", self.PRI_MOD),
+                    ast.Mult: ("*", self.PRI_MULT),
+                    ast.RShift: (">>", self.PRI_SHIFT),
+                    ast.Sub: ("-", self.PRI_ADD),
+                }[type(node.op)]
+            except KeyError:
+                try:
+                    if isinstance(node.op, ast.Div):
+                        # special case for common error
+                        raise TranspilerError("unsupported operator / (use // for integer division)", ast_node=node)
+                    op_str = {
+                        ast.MatMult: "@",
+                        ast.Div: "/",
+                        ast.Pow: "**"
+                    }[type(node.op)]
+                    raise TranspilerError(f"unsupported binary operator {op_str}", ast_node=node)
+                except KeyError:
+                    raise TranspilerError("unknown binary operator", ast_node=node)
             left, aux_st, _ = self.compile_expr(node.left, context, priority)
             aux_statements += aux_st
             right, aux_st, _ = self.compile_expr(node.right, context, priority)
@@ -623,7 +653,7 @@ end
                     arg = node.args[i]
                     code, aux_st, is_boolean = self.compile_expr(arg, context, self.PRI_COMMA)
                     aux_statements += aux_st
-                    aux_statements += f"""{function_def.var_str(arg_def.arg)} = {code}
+                    aux_statements += f"""{function_def.var_str(arg_def.arg, ast_node=arg)} = {code}
 """
                 # call
                 aux_statements += f"""callsub {fun_name}
@@ -760,7 +790,7 @@ end
                 raise TranspilerError(f"unknown variable '{node.id}'", node)
             if isinstance(var_array_size, int):
                 raise TranspilerError(f"list variable '{node.id}' used in expression", node)
-            code = context.var_str(node.id)
+            code = context.var_str(node.id, ast_node=node)
         elif isinstance(node, ast.Subscript):
             name = self.decode_attr(node.value)
             var_array_size = context.var_array_size(name)
@@ -780,7 +810,7 @@ end
 """
                 index_value = context.tmp_var_str(tmp_offset)
             aux_statements += aux_st
-            code = f"{context.var_str(name)}[{index_value}]"
+            code = f"{context.var_str(name, ast_node=node)}[{index_value}]"
         elif isinstance(node, ast.UnaryOp):
             op = node.op
             if isinstance(op, ast.UAdd):
@@ -846,7 +876,7 @@ end
             if len(node.targets) != 1:
                 raise TranspilerError("unsupported assignment to multiple targets", node)
             target, index = self.decode_target(node.targets[0])
-            target_str = context.var_str(target, True)
+            target_str = context.var_str(target, True, ast_node=node.targets[0])
             if index is None:
                 # declare target
                 list_len = context.list_length(node.value)
@@ -867,7 +897,7 @@ end
                         return f"""{target_str} = {module_value}
 """
                     else:
-                        return f"""{target_str} = {context.var_str(name_right)}
+                        return f"""{target_str} = {context.var_str(name_right, ast_node=node)}
 """
                 else:
                     context.declare_var(target, ast_node=node)
@@ -919,7 +949,7 @@ end
                 ast.Sub: "-",
             }[type(node.op)]
             target, index = self.decode_target(node.target)
-            target_str = context.var_str(target)
+            target_str = context.var_str(target, ast_node=node)
             value, aux_statements, is_boolean = self.compile_expr(node.value, context, self.PRI_NUMERIC)
             code += aux_statements
             if index is not None:
@@ -1041,7 +1071,7 @@ end
                 raise TranspilerError("for loop with unsupported iterator (not range)", node)
             range_args = node.iter.args
             target = self.decode_attr(node.target)
-            target_str = context.var_str(target)
+            target_str = context.var_str(target, ast_node=node.target)
             context.declare_var(target, ast_node=node)
             if len(range_args) == 1:
                 # for var in range(a): ...
@@ -1112,12 +1142,22 @@ while {target_str} * {context.tmp_var_str(tmp_offset + 1)} < {context.tmp_var_st
 """
             body = self.compile_node_array(node.body, context)
             code += body
+            end_count = 1  # number of final "end", which depends on nested else if
             while len(node.orelse) == 1 and isinstance(node.orelse[0], ast.If):
                 # "if" node as single element of orelse: elif
                 node = node.orelse[0]
                 test_value, aux_statements, is_boolean = self.compile_expr(node.test, context, self.PRI_LOW)
-                code += aux_statements
-                code += f"""elseif {test_value}{"" if is_boolean else " != 0"} then
+                if aux_statements:
+                    # computation of condition in separate statements between else and if
+                    code += f"""else
+"""
+                    code += aux_statements
+                    code += f"""if {test_value}{"" if is_boolean else " != 0"} then
+"""
+                    end_count += 1
+                else:
+                    # no separate statement to compute condition between else and if: elseif is fine
+                    code += f"""elseif {test_value}{"" if is_boolean else " != 0"} then
 """
                 body = self.compile_node_array(node.body, context)
                 code += body
@@ -1127,7 +1167,7 @@ while {target_str} * {context.tmp_var_str(tmp_offset + 1)} < {context.tmp_var_st
 """
                 body = self.compile_node_array(node.orelse, context)
                 code += body
-            code += """end
+            code += end_count * """end
 """
             return code
         if isinstance(node, ast.Import):
@@ -1236,7 +1276,9 @@ return
         # function return type isn't known yet (context.has_ret_value = None doesn't raise exceptions)
         for fun_name in context_top.functions:
             fun_print_format_string_next_id = self.print_format_string_next_id
+            context_top.functions[fun_name].collect_local_variables = True
             _ = self.compile_node_array(context_top.functions[fun_name].function_def.body, context_top.functions[fun_name])
+            context_top.functions[fun_name].collect_local_variables = False
             # set functions without return statements to void
             context_top.functions[fun_name].freeze_return_type()
             self.reset_transpile_phase(fun_print_format_string_next_id)
