@@ -5,7 +5,9 @@
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from tdmclient import TDMZeroconfBrowser, TDMConnection, FlatBuffer, ThymioFB
+from tdmclient import TDMZeroconfBrowser, TDMConnection
+from tdmclient import FlatBuffer, ThymioFB
+from tdmclient.clientnode import ClientNode
 
 
 class DisconnectedError(Exception):
@@ -16,22 +18,31 @@ class DisconnectedError(Exception):
 
 class Client(ThymioFB):
 
-    def __init__(self, tdm_addr=None, tdm_port=None, **kwargs):
+    DEFAULT_TDM_PORT = 8596
+
+    def __init__(self,
+                 zeroconf=None, tdm_addr=None, tdm_port=None, password=None,
+                 **kwargs):
 
         super(Client, self).__init__(**kwargs)
 
+        if zeroconf is None:
+            # use zeroconf if tdm_port isn't specified
+            zeroconf = tdm_port is None
         self.tdm_addr = tdm_addr
         self.tdm_port = tdm_port
         self.tdm = None
 
-        def on_change(is_added, addr, port, ws_port):
+        def on_zc_change(is_added, addr, port, ws_port):
             if is_added and self.tdm_addr is None:
                 if self.debug >= 1:
                     print(f"Zeroconf: TDM {addr}:{port} on")
-                self.tdm_addr = addr
-                self.tdm_port = port
+                if tdm_addr is None:
+                    self.tdm_addr = addr
+                if tdm_port is None:
+                    self.tdm_port = port
                 self.connect()
-                self.send_handshake()
+                self.send_handshake(password)
             elif not is_added and addr == self.tdm_addr and port == self.tdm_port:
                 if self.debug >= 1:
                     print(f"Zeroconf: TDM {addr}:{port} off")
@@ -39,19 +50,19 @@ class Client(ThymioFB):
                 self.tdm_addr = None
                 self.tdm_port = None
 
-        if tdm_port is None:
-            # no port provided: rely on zeroconf
-            self.zc = TDMZeroconfBrowser(on_change)
+        if zeroconf:
+            self.zc = TDMZeroconfBrowser(on_zc_change)
         else:
-            # port provided: use it without zeroconf
             self.zc = None
+            if self.tdm_port is None:
+                self.tdm_port = self.DEFAULT_TDM_PORT
             if tdm_addr is None:
                 # localhost by default
                 self.tdm_addr = "127.0.0.1"
             if self.debug >= 1:
                 print(f"TDM {self.tdm_addr}:{self.tdm_port}")
             self.connect()
-            self.send_handshake()
+            self.send_handshake(password)
 
     def close(self):
         if self.zc is not None:
@@ -80,7 +91,7 @@ class Client(ThymioFB):
             # check decoding
             fb2 = FlatBuffer()
             fb2.parse(b, self.SCHEMA)
-            fb2.dump()
+            fb2.dump("Send")
 
         if self.tdm is None:
             if ignore_disconnected_error:
@@ -99,10 +110,10 @@ class Client(ThymioFB):
 
         self.send_packet(encoded_fb)
 
-    def send_handshake(self):
+    def send_handshake(self, password=None):
         if self.debug >= 1:
             print("send handshake")
-        self.send_packet(self.create_msg_handshake())
+        self.send_packet(self.create_msg_handshake(password))
 
     def shutdown_tdm(self, **kwargs):
         """Send a shutdown request. No reply should be expected.
