@@ -6,6 +6,7 @@
 # SPDX-License-Identifier: BSD-3-Clause
 
 from tdmclient import ThymioFB, ClientNode
+from tdmclient.clientasync import NodeLockError
 import types
 
 
@@ -57,7 +58,7 @@ class ClientAsyncNode(ClientNode):
 
         result = yield from self.lock_node()
         if result is not None:
-            raise Exception("Node lock error")
+            raise NodeLockError(self.status)
         return self
 
     def __enter__(self):
@@ -147,7 +148,7 @@ class ClientAsyncNode(ClientNode):
             self.watch_flags |= flags
             result = yield from self.thymio.send_msg_and_get_result(
                 lambda notify:
-                    self.watch_node(flags, request_id_notify=notify)
+                    self.watch_node(self.watch_flags, request_id_notify=notify)
             )
             return result
 
@@ -159,6 +160,6 @@ class ClientAsyncNode(ClientNode):
             self.watch_flags &= ~flags
             result = yield from self.thymio.send_msg_and_get_result(
                 lambda notify:
-                    self.watch_node(flags, request_id_notify=notify)
+                    self.watch_node(self.watch_flags, request_id_notify=notify)
             )
             return result
