@@ -345,6 +345,9 @@ class FlatBuffer:
             enc += bytes([0 for i in range(4 - len(enc) % 4)])
             return value, enc, False
         elif schema[0] == "*":
+            if value is None:
+                # default value
+                return value, None, False
             enc = FlatBuffer.encode_32(len(value))
             if schema[1] == "u":
                 if type(value) is bytes:
@@ -459,7 +462,7 @@ class FlatBuffer:
         else:
             raise TypeError("unsupported type in FlatBuffer")
 
-    def dump(self):
+    def dump(self, msg=None):
 
         def dump_value(value):
             if type(value) == int:
@@ -492,6 +495,8 @@ class FlatBuffer:
             else:
                 print("unknown", value)
 
+        if msg is not None:
+            print(f"{msg}:")
         dump_value(self.root)
 
 
