@@ -7,7 +7,6 @@
 
 import code
 import ast
-import sys
 import re
 
 from tdmclient import ClientAsync, ArrayCache
@@ -35,6 +34,7 @@ class TDMConsole(code.InteractiveConsole):
         self.client = None
         self.node = None
 
+        # self.event_data_dict[node_id][event_name] = list of event data
         self.event_data_dict = {}
 
         def onevent(fun):
@@ -119,7 +119,7 @@ class TDMConsole(code.InteractiveConsole):
         def run(src=None, *,
                 language="python",
                 wait=None,
-                robot_id=None, robot_name=None):
+                **kwargs):
             """Run program obtained by robot_code on the robot. By default, wait
             to process events until "_exit" is received (call to "exit()" in the
             robot's program), or return immediately if the program doesn't send
@@ -133,6 +133,7 @@ class TDMConsole(code.InteractiveConsole):
                 source code is passed in a string)
                 robot_id: robot id, to run the program on a specific robot
                 robot_name: robot name, to run the program on a specific robot
+                robot_index: robot index (0=first=default, 1=second etc.)
             """
 
             if src is None and language != "python":
@@ -141,21 +142,22 @@ class TDMConsole(code.InteractiveConsole):
                 src = robot_code()
             # compile, load, run, and set scratchpad without checking the result
             try:
-                self.run_program(src, language=language, wait=wait,
-                                 node_id=robot_id, node_name=robot_name)
+                node = self.find_robot(**kwargs) or self.node
+                self.run_program(src, [node], language=language, wait=wait)
             except KeyboardInterrupt:
                 # avoid long exception message with stack trace
                 print("Interrupted")
 
-        def stop(robot_id=None, robot_name=None):
+        def stop(**kwargs):
             """Stop the program running on the robot.
 
             Keyword arguments:
                 robot_id: robot id, to run the program on a specific robot
                 robot_name: robot name, to run the program on a specific robot
+                robot_index: robot index (0=first=default, 1=second etc.)
             """
-            self.stop_program(discard_output=True,
-                              node_id=robot_id, node_name=robot_name)
+            node = self.find_robot(**kwargs) or self.node
+            self.stop_program(node, discard_output=True)
 
         def get_var(*args):
             """Get robot variables passed as a set or list of names and
@@ -175,27 +177,51 @@ class TDMConsole(code.InteractiveConsole):
                 self.send_variable(name_py, kwargs[name_py])
             self.flush_variables()
 
-        def clear_event_data(event_name=None):
+        def clear_event_data(event_name=None, **kwargs):
             """Clear all or named event data.
-            """
-            self.clear_event_data(event_name)
 
-        def get_event_data(event_name=None):
+            Argument:
+                event_name: event name (default: all events)
+            Keyword argument:
+                robot_id: robot id, to run the program on a specific robot
+                robot_name: robot name, to run the program on a specific robot
+                robot_index: robot index (0=first=default, 1=second etc.)
+            """
+            node = self.find_robot(**kwargs)
+            self.clear_event_data(event_name, node=node)
+
+        def get_event_data(event_name=None, **kwargs):
             """Get list of event data received until now.
-            """
-            return self.get_event_data(event_name)
 
-        def send_event(event_name, *args):
+            Argument:
+                event_name: event name (default: dict of all event lists)
+            Keyword argument:
+                robot_id: robot id, to run the program on a specific robot
+                robot_name: robot name, to run the program on a specific robot
+                robot_index: robot index (0=first=default, 1=second etc.)
+            """
+            node = self.find_robot(**kwargs)
+            return self.get_event_data(event_name, node=node)
+
+        def send_event(event_name, *args, **kwargs):
             """Send a custom event to the robot. Arguments can be numbers,
             booleans and/or arrays.
+
+            Keyword arguments:
+                robot_id: robot id, to run the program on a specific robot
+                robot_name: robot name, to run the program on a specific robot
+                robot_index: robot index (0=first=default, 1=second etc.)
             """
+
             # flatten args
             data = [
                 item
                 for arg in args
                 for item in (arg if isinstance(arg, list) else [arg])
             ]
-            self.node.send_send_events({event_name: data})
+
+            node = self.find_robot(**kwargs) or self.node
+            node.send_send_events({event_name: data})
 
         self.functions = {
             "onevent": onevent,
@@ -289,17 +315,23 @@ class TDMConsole(code.InteractiveConsole):
         transpiler.transpile()
         return transpiler
 
-    def clear_event_data(self, event_name=None):
-        if event_name is None:
-            self.event_data_dict = {}
-        elif event_name in self.event_data_dict:
-            del self.event_data_dict[event_name]
+    def clear_event_data(self, event_name=None, node=None):
+        node_id = (self.node if node is None else node).id_str
+        if node_id in self.event_data_dict:
+            if event_name is None:
+                del self.event_data_dict[node_id]
+            elif event_name in self.event_data_dict[node_id]:
+                del self.event_data_dict[node_id][event_name]
 
-    def get_event_data(self, event_name=None):
-        if event_name is None:
-            return self.event_data_dict
+    def get_event_data(self, event_name=None, node=None):
+        node_id = (self.node if node is None else node).id_str
+        if node_id in self.event_data_dict:
+            if event_name is None:
+                return self.event_data_dict[node_id]
+            else:
+                return self.event_data_dict[node_id][event_name] if event_name in self.event_data_dict[node_id] else []
         else:
-            return self.event_data_dict[event_name] if event_name in self.event_data_dict else []
+            return {} if event_name is None else []
 
     def reset_sync_var(self):
         self.sync_var = self.sync_var_vm.copy()
@@ -307,12 +339,12 @@ class TDMConsole(code.InteractiveConsole):
         for name in self.node.var:
             self.sync_var.add(name)
 
-    def process_events(self, on_event_data=None):
+    def process_events(self, on_event_data=None, all_nodes=False):
         """Listen to events sent by the program running on the robot and process
         them until _exit is received.
 
         Argument:
-            on_event_data -- func(event_name) called when new data is received
+            on_event_data -- func(node, event_name) called when new data is received
         """
 
         exit_received = None  # or exit code once received
@@ -330,11 +362,13 @@ class TDMConsole(code.InteractiveConsole):
                     print(print_str)
                 else:
                     if len(event_data) > 0:
-                        if event_name not in self.event_data_dict:
-                            self.event_data_dict[event_name] = []
-                        self.event_data_dict[event_name].append(event_data)
+                        if node.id_str not in self.event_data_dict:
+                            self.event_data_dict[node.id_str] = {}
+                        if event_name not in self.event_data_dict[node.id_str]:
+                            self.event_data_dict[node.id_str][event_name] = []
+                        self.event_data_dict[node.id_str][event_name].append(event_data)
                         if on_event_data is not None:
-                            on_event_data(event_name)
+                            on_event_data(node, event_name)
 
         def wake():
             return exit_received is not None
@@ -342,33 +376,125 @@ class TDMConsole(code.InteractiveConsole):
         self.client.clear_event_received_listeners()
         self.client.add_event_received_listener(on_event_received)
         try:
-            ClientAsync.aw(self.node.watch(events=True))
+            if all_nodes:
+                for node in self.client.nodes:
+                    ClientAsync.aw(node.watch(events=True))
+            else:
+                ClientAsync.aw(self.node.watch(events=True))
             ClientAsync.aw(self.client.sleep(wake=wake))
-            self.stop_program(discard_output=True)
+            self.stop_program(self.node, discard_output=True)
             if exit_received:
                 print(f"Exit, status={exit_received}")
         finally:
+            if all_nodes:
+                for node in self.client.nodes:
+                    ClientAsync.aw(node.watch(events=False))
+            else:
+                ClientAsync.aw(self.node.watch(events=False))
             self.client.clear_event_received_listeners()
 
-    def run_program(self, src,
-                    language="aseba", wait=False, import_thymio=True,
-                    node_id=None, node_name=None):
-        node = self.node
-        if node_id is not None or node_name is not None:
-            # could be another node we lock just for this call
-            node = self.client.first_node(node_id=node_id, node_name=node_name)
-            if node != self.node:
-                ClientAsync.aw(node.lock())
+    def find_robot(self,
+                  robot_id=None, robot_name=None,
+                  robot_index=None):
+        """Find the node specified by id, name or index.
+        """
+        if robot_index is not None:
+            return self.client.nodes[robot_index]
+        elif robot_id is not None or robot_name is not None:
+            return self.client.first_node(node_id=robot_id,
+                                          node_name=robot_name)
 
-        try:
-            print_statements = []
+    def lock_robots(self, nodes):
+        """Target the robot referenced by the key arguments, lock them if
+        they aren't the default node, and return them; must be used with "with".
+        """
+
+        class Robot:
+            """Utility class to be used with "with", to lock the nodes which
+            aren't the default one and return them.
+            """
+
+            def __init__(self,
+                         console,
+                         default_node,
+                         nodes):
+                self.default_node = default_node
+                self.nodes = nodes
+                for node in nodes:
+                    if node != default_node:
+                        ClientAsync.aw(node.lock())
+
+            def __enter__(self):
+                return self.nodes
+
+            def __exit__(self, type, value, traceback):
+                for node in nodes:
+                    if node != self.default_node:
+                        node.unlock()
+
+        return Robot(self, self.node, nodes)
+
+
+    def run_program(self, src,
+                    nodes=None,
+                    language="aseba", wait=False, import_thymio=True):
+        if nodes is None:
+            nodes = [self.node]
+
+        running_nodes = set()
+
+        # exit_received[node] = exit code once received
+        exit_received = {}
+        # print_statements[node][print_id] = (print_format, print_num_args)
+        print_statements = {}
+
+        def on_event_received(node, event_name, event_data):
+            if self.output_enabled:
+                if event_name == "_exit":
+                    exit_received[node] = event_data[0]
+                    if event_data[0]:
+                        exit_str = f"Exit, status={event_data[0]}"
+                        if len(nodes) > 1:
+                            # multiple nodes: add prefix
+                            exit_str = f"[R{nodes.index(node)}] " + exit_str
+                        print(exit_str)
+                    self.stop_program(node, discard_output=True)
+                    running_nodes.remove(node)
+                elif event_name == "_print":
+                    print_id = event_data[0]
+                    print_format, print_num_args = print_statements[node][print_id]
+                    print_args = tuple(event_data[1 : 1 + print_num_args])
+                    print_str = print_format % print_args
+                    if len(nodes) > 1:
+                        # multiple nodes: add prefix
+                        print_str = f"[R{nodes.index(node)}] " + print_str
+                    print(print_str)
+                else:
+                    if len(event_data) > 0:
+                        if node.id_str not in self.event_data_dict:
+                            self.event_data_dict[node.id_str] = {}
+                        if event_name not in self.event_data_dict[node.id_str]:
+                            self.event_data_dict[node.id_str][event_name] = []
+                        self.event_data_dict[node.id_str][event_name].append(event_data)
+
+        def on_vm_state_changed(node, state, line, error, error_msg):
+            if error != ClientAsync.ERROR_NO_ERROR:
+                exit_received[node] = f"vm error {error}"
+            if error_msg:
+                print(f"{error_msg} (line {line}{' in Aseba' if language != 'aseba' else ''})")
+
+        def run_node(node):
+            """Compile, configure node, load and start program on a node.
+            Return True if the node requires waiting.
+            """
+            print_statements[node] = []
             events = []
             if language == "python":
                 # transpile from Python to Aseba
                 transpiler = self.transpile(src, import_thymio)
-                src = transpiler.get_output()
-                print_statements = transpiler.print_format_strings
-                if len(print_statements) > 0:
+                src_aseba = transpiler.get_output()
+                print_statements[node] = transpiler.print_format_strings
+                if len(print_statements[node]) > 0:
                     events.append(("_print", 1 + transpiler.print_max_num_args))
                 if transpiler.has_exit_event:
                     events.append(("_exit", 1))
@@ -378,83 +504,63 @@ class TDMConsole(code.InteractiveConsole):
                     events.append((event_name, transpiler.events_out[event_name]))
                 if len(events) > 0:
                     events = ClientAsync.aw(node.filter_out_vm_events(events))
+                if len(events) > 0:
                     ClientAsync.aw(node.register_events(events))
-            elif language != "aseba":
+            elif language == "aseba":
+                src_aseba = src
+            else:
                 raise Exception(f"Unsupported language {language}")
-            # compile, load, run, and set scratchpad without checking the result
-            error = ClientAsync.aw(node.compile(src))
+            error = ClientAsync.aw(node.compile(src_aseba))
             if error is not None:
                 raise Exception(error["error_msg"])
-            self.client.clear_event_received_listeners()
-            exit_received = None  # or exit code once received
+            node.send_set_scratchpad(src_aseba)
+            wait_for_node = wait
             if wait is None:
-                # wait if there are events to receive
-                wait = len(events) > 0
-            if wait:
-                if len(events) > 0:
-                    def on_event_received(node, event_name, event_data):
-                        if self.output_enabled:
-                            if event_name == "_exit":
-                                nonlocal exit_received
-                                exit_received = event_data[0]
-                            elif event_name == "_print":
-                                print_id = event_data[0]
-                                print_format, print_num_args = print_statements[print_id]
-                                print_args = tuple(event_data[1 : 1 + print_num_args])
-                                print_str = print_format % print_args
-                                print(print_str)
-                            else:
-                                if len(event_data) > 0:
-                                    if event_name not in self.event_data_dict:
-                                        self.event_data_dict[event_name] = []
-                                    self.event_data_dict[event_name].append(event_data)
-                    self.client.add_event_received_listener(on_event_received)
-                def on_vm_state_changed(node, state, line, error, error_msg):
-                    if error != ClientAsync.ERROR_NO_ERROR:
-                        nonlocal exit_received
-                        exit_received = f"vm error {error}"
-                    if error_msg:
-                        print(f"{error_msg} (line {line}{' in Aseba' if language != 'aseba' else ''})")
-                self.client.add_vm_state_changed_listener(on_vm_state_changed)
+                # default: wait if there are events to receive
+                wait_for_node = len(events) > 0
+            if wait_for_node:
                 ClientAsync.aw(node.watch(events=True, vm_state=True))
-            self.reset_sync_var()
             error = ClientAsync.aw(node.run())
             if error is not None:
                 raise Exception(f"Error {error['error_code']}")
-            node.send_set_scratchpad(src)
-            if wait:
-                try:
-                    def wake():
-                        return exit_received is not None
-                    ClientAsync.aw(self.client.sleep(wake=wake))
-                    self.stop_program(discard_output=True)
-                    if exit_received:
-                        print(f"Exit, status={exit_received}")
-                finally:
-                    self.client.clear_event_received_listeners()
-        finally:
-            if node != self.node:
-                node.unlock()
+            return wait_for_node
 
-    def stop_program(self, discard_output=False,
-                     node_id=None, node_name=None):
-        node = self.node
-        if node_id is not None or node_name is not None:
-            # could be another node we lock just for this call
-            node = self.client.first_node(node_id=node_id, node_name=node_name)
-            if node != self.node:
-                ClientAsync.aw(node.lock())
+        self.reset_sync_var()
+        self.client.clear_event_received_listeners()
+        self.client.add_event_received_listener(on_event_received)
+        self.client.add_vm_state_changed_listener(on_vm_state_changed)
+        wait_for_nodes = False
+        with self.lock_robots(nodes) as nodes_l:
+            # transpile, compile, load, set scratchpad, and run
+            for node in nodes_l:
+                wait_for_node = run_node(node)
+                wait_for_nodes = wait_for_nodes or wait_for_node
+                running_nodes.add(node)
 
-        output_enabled_orig = self.output_enabled
-        self.output_enabled = not discard_output
-        try:
-            error = ClientAsync.aw(node.stop())
-            if error is not None:
-                raise Exception(f"Error {error['error_code']}")
-        finally:
-            self.output_enabled = output_enabled_orig
-            if node != self.node:
-                node.unlock()
+        # wait until all nodes have exited
+        if wait_for_nodes:
+            try:
+                def wake():
+                    # True when all nodes have exited
+                    return len(exit_received) >= len(nodes)
+                ClientAsync.aw(self.client.sleep(wake=wake))
+            finally:
+                # stop nodes still running
+                for node in running_nodes:
+                    self.stop_program(node, discard_output=True)
+                self.client.clear_event_received_listeners()
+                self.client.clear_vm_state_changed_listener()
+
+    def stop_program(self, node, discard_output=False):
+        with self.lock_robots({node}) as nodes_l:
+            output_enabled_orig = self.output_enabled
+            self.output_enabled = not discard_output
+            try:
+                error = ClientAsync.aw(node.stop())
+                if error is not None:
+                    raise Exception(f"Error {error['error_code']}")
+            finally:
+                self.output_enabled = output_enabled_orig
 
     def from_python_name(self, name_py):
         # replace underscores with dots, except first underscore, for node variables
@@ -634,6 +740,8 @@ class TDMConsole(code.InteractiveConsole):
                 do_node(node.lower)
                 do_node(node.upper)
                 do_node(node.step)
+            elif isinstance(node, ast.Starred):
+                do_node(node.value)
             elif isinstance(node, ast.Subscript):
                 do_node(node.value)
                 do_node(node.slice)
