@@ -14,8 +14,6 @@ import socket
 import io
 import threading
 import queue
-import time
-from typing import Awaitable, Callable, List, Optional, Set, Tuple
 
 
 class InputThread(threading.Thread):
@@ -51,7 +49,9 @@ class InputThread(threading.Thread):
                 if not self.running:
                     raise Exception("closing")
                 packet_len = self.read_uint32()
-                packet = self.io.read(packet_len)
+                packet = b""
+                while len(packet) < packet_len:
+                    packet += self.io.read(packet_len - len(packet))
             return packet
         except Exception as error:
             self.comm_error = error
