@@ -11,7 +11,7 @@ Author: Yves Piguet, EPFL
 """
 
 
-from tdmclient import FlatBuffer, Union, Table
+from tdmclient import FlatBuffer, Union
 
 
 class Listener:
@@ -380,7 +380,24 @@ class ThymioFB(Listener):
         // root object: union AnyMessage
         U(
             // ConnectionHandshake
-            T(iiibb)
+            T(
+                // minProtocolVersion: uint
+                i
+                // protocolVersion: uint
+                i
+                // maxMessageSize: uint
+                i
+                // token: [ubyte]
+                *u
+                // localhostPeer: bool
+                b
+                // ws_port: uint2
+                2
+                // uuid: NodeId
+                T(*u)
+                // password: string
+                s
+            )
             // DeviceManagerShutdownRequest
             T(i)
             // RequestListOfNodes
@@ -418,7 +435,29 @@ class ThymioFB(Listener):
                 )
             )
             // NodeAsebaVMDescription
-            T(iT(*u)iii*T(2si)*T(2ss)*T(2ss*T(si)))
+            T(
+                // request_id
+                i
+                // node_id
+                T(*u)
+                // bytecode_size
+                i
+                // data_size
+                i
+                // stack_size
+                i
+                // variables
+                *T(2si)
+                // events
+                *T(2ss)
+                // functions
+                *T(
+                    2
+                    s
+                    s
+                    *T(si)
+                )
+            )
             // RequestCompleted
             T(i)
             // Error
@@ -483,10 +522,19 @@ class ThymioFB(Listener):
 
         return encoded_fb
 
-    def create_msg_handshake(self):
+    def create_msg_handshake(self, password=None):
         return self.create_message((
             ThymioFB.MESSAGE_TYPE_CONNECTION_HANDSHAKE,
-            ()
+            (
+                1,  # default minProtocolVersion
+                1,  # default protocolVersion
+                102400,  # default maxMessageSize
+                None,  # default token
+                False,  # default localhostPeer
+                0,  # default ws_port
+                None,  # default NodeId
+                password,
+            ) if password is not None else ()
         ), ThymioFB.SCHEMA)
 
     def create_msg_device_manager_shutdown_request(self, **kwargs):
@@ -584,7 +632,7 @@ class ThymioFB(Listener):
         fb = FlatBuffer()
         fb.parse(msg, ThymioFB.SCHEMA)
         if self.debug >= 2:
-            fb.dump()
+            fb.dump("Receive")
         if type(fb.root) is Union:
             if fb.root.union_type == self.MESSAGE_TYPE_PING:
                 pass
